/*
 * SPDX-FileCopyrightText: 2015 Luca Beltrame <lbeltrame@kde.org>
 *
 * This file is part of Danbooru Client.
 *
 *SPDX-License-Identifier: GPL-3.0-or-later
 */

#include "danbooruconnectwidget.h"
#include "libdanbooru/danbooruservice.h"

#include <QCryptographicHash>
#include "danbooru_client_debug.h"

#include <KWallet>

using KWallet::Wallet;

namespace Danbooru
{

const QMap< QUrl, QString > initBoardSalts()
{

    QMap< QUrl, QString > boardSalts;

    boardSalts.insert(QUrl(QStringLiteral("http://konachan.com")),
                      QStringLiteral("So-I-Heard-You-Like-Mupkids-?--%1--"));
    boardSalts.insert(QUrl(QStringLiteral("http://konachan.net")),
                      QStringLiteral("So-I-Heard-You-Like-Mupkids-?--%1--"));
    boardSalts.insert(QUrl(QStringLiteral("http://yande.re")),
                      QStringLiteral("choujin-steiner--%1--"));
    boardSalts.insert(QUrl(QStringLiteral("http://danbooru.donmai.us")),
                      QStringLiteral("choujin-steiner--%1--"));

    return boardSalts;
}

const QMap<QUrl, QString> DanbooruConnectWidget::boardSalts = initBoardSalts();

DanbooruConnectWidget::DanbooruConnectWidget(QVector< QUrl > urlList,
        QWidget *parent):
    QWidget(parent),
    m_wallet(0)

{
    setupUi(this);

    danbooruUrlComboBox->setFocus();
    closeButton->setIcon(QIcon::fromTheme(QStringLiteral("dialog-close")));
    closeButton->setToolTip(i18n("Close dialog and discard changes"));
    userLineEdit->setClearButtonEnabled(true);
    passwdLineEdit->setClearButtonEnabled(true);
    passwdLineEdit->setEchoMode(QLineEdit::Password);

    if (anonCheckBox->isChecked()) {
        userLineEdit->setEnabled(false);
        passwdLineEdit->setEnabled(false);
    }

    danbooruUrlComboBox->clear();

    Q_FOREACH(const auto& item, urlList) {
        danbooruUrlComboBox->insertUrl(urlList.indexOf(item), item);
    }

    m_wallet = Wallet::openWallet(Wallet::NetworkWallet(), 0,
                                  Wallet::Asynchronous
                                 );

    connect(m_wallet, &KWallet::Wallet::walletOpened, this, &DanbooruConnectWidget::checkWallet);

    connect(danbooruUrlComboBox, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &DanbooruConnectWidget::getWalletData);

    connect(anonCheckBox, &QCheckBox::stateChanged, this, &DanbooruConnectWidget::toggleLineEdits);

    connect(buttonBox, &QDialogButtonBox::accepted, this, &DanbooruConnectWidget::accept);
    connect(closeButton, &QPushButton::clicked, [this]() {
        Q_EMIT(rejected());
    });

}

DanbooruConnectWidget::~DanbooruConnectWidget()
{
    delete m_wallet;
}

void DanbooruConnectWidget::checkWallet(bool result)
{
    if (!result) {
        return;
    }

    if (!m_wallet->hasFolder(Wallet::PasswordFolder())) {
        m_wallet->createFolder(Wallet::PasswordFolder());
    }

    getWalletData();
}

void DanbooruConnectWidget::getWalletData()
{
    if (!m_wallet) {
        return;
    }

    QMap<QString, QString> valueMap;
    QString key = danbooruUrlComboBox->currentText();

    if (m_wallet->hasEntry(key)) {

        if (m_wallet->readMap(key, valueMap) != 0) {
            return;
        }

        m_username = valueMap[QStringLiteral("username")];

        QString hashedPassword;

        // TODO: Handle the case where the "salt" is not known
        hashedPassword = boardSalts.value(key);

        hashedPassword = hashedPassword.arg(valueMap[QStringLiteral("password")]);
        hashedPassword = QCryptographicHash::hash(hashedPassword.toUtf8(),
                         QCryptographicHash::Sha1).toHex();

        m_password = hashedPassword;
        userLineEdit->setText(m_username);
        passwdLineEdit->setText(valueMap[QStringLiteral("password")]);

    }
}

void DanbooruConnectWidget::toggleLineEdits(int state)
{
    if (state == Qt::Unchecked) {
        userLineEdit->setEnabled(true);
        passwdLineEdit->setEnabled(true);
    } else if (state == Qt::Checked) {
        userLineEdit->setEnabled(false);
        passwdLineEdit->setEnabled(false);
    }
}

void DanbooruConnectWidget::setBoards(const QVector<QUrl> &urlList)
{

    danbooruUrlComboBox->clear();

    Q_FOREACH(const auto& item, urlList) {
        danbooruUrlComboBox->insertUrl(urlList.indexOf(item), item);
    }

}

QUrl DanbooruConnectWidget::boardUrl() const
{
    return m_boardUrl;
}

QString DanbooruConnectWidget::username() const
{
    return m_username;
}

QString DanbooruConnectWidget::password() const
{
    return m_password;
}

void DanbooruConnectWidget::accept()

{

    QString currentBoard = danbooruUrlComboBox->currentText();

    if (!userLineEdit->text().isEmpty() &&
            !passwdLineEdit->text().isEmpty() &&
            !anonCheckBox->isChecked()) {

        if (m_wallet && !m_wallet->hasEntry(currentBoard)) {
            QMap<QString, QString> dataMap;
            dataMap.insert(QStringLiteral("username"), m_username);
            dataMap.insert(QStringLiteral("password"), passwdLineEdit->text());
            m_wallet->writeMap(m_boardUrl.url(), dataMap);
        }

        // Only calculate if we haven't set a password from the wallet already
        if (m_password.isEmpty()) {

            QString hashedPassword = boardSalts.value(danbooruUrlComboBox->currentText());
            hashedPassword = hashedPassword.arg(currentBoard);
            hashedPassword = QCryptographicHash::hash(hashedPassword.toUtf8(),
                             QCryptographicHash::Sha1).toHex();
            m_password = hashedPassword;
        }
    }

    m_boardUrl = QUrl::fromUserInput(danbooruUrlComboBox->currentText());
    Q_EMIT(accepted());
}

bool DanbooruConnectWidget::isAnonymous() const
{

    return anonCheckBox->isChecked();

}

} // namespace Danbooru
