/*
 * SPDX-FileCopyrightText: 2015 Luca Beltrame <lbeltrame@kde.org>
 *
 * This file is part of Danbooru Client.
 *
 *SPDX-License-Identifier: GPL-3.0-or-later
 */

#ifndef UTILS_H
#define UTILS_H

#include "danboorupost.h"

// Qt

#include <QStringList>
#include <QVariantMap>
#include <QUrl>


/**
 * @brief Commmon utilities for Danbooru classes.
 * @file utils.h
 *
 **/

namespace Danbooru
{

typedef QMap<QString, QString> dictMap;

/** @brief Generate a request URL for a Danbooru board.
 *
 * Given an URL and an API path, this function builds a
 * proper URL which is used for the specific API operation.
 * The processing follows the "normal" way of encoding URLs.
 *
 * @param url The board URL.
 * @param path The API path of the call to use
 * @param username The username to supply (optional)
 * @param password The password to use (optional)
 * @param parameters A map of key,values representing the parameters
 *                   to use.
 * @param tags The tags to supply (optional).
 *
 * @return A constructed URL to be used for a Danbooru API call.
 * @author Luca Beltrame (lbeltrame@kde.org)
 *
 *
 **/
QUrl requestUrl(const QUrl &url, const QString &username = QString(),
                const QString &password = QString(),
                const dictMap &parameters = dictMap(),
                const QStringList &tags = QStringList());

/** @brief Generate a request URL for a Danbooru board.
 *
 * This is an overloaded function provided for convenience.
 *
 *
 * @param url The board URL.
 * @param path The API path of the call to use
 * @param username The username to supply (optional)
 * @param password The password to use (optional)
 * @param parameters A map of key,values representing the parameters
 *                   to use.
 *
 * @return A constructed URL to be used for a Danbooru API call.
 * @author Luca Beltrame (lbeltrame@kde.org)
 *
 *
 **/


QList<QVariant> parseDanbooruResult(QByteArray data, QString xlmElement,
                                    bool *result, ApiType=ApiType::MoeBooru);
QVariant parseDanbooruResult(QByteArray data, bool *result);

/**
 * @brief Check if a post can be allowed.
 *
 * This convenience function checks if a DanbooruPost is not allowed, either because it
 * contains blacklisted tags, or because it has a higher rating than allowed.
 *
 * @param post A DanbooruPost pointer.
 * @param blacklist A QSet containing unwanted tag names as QStrings.
 * @param maxRating The maximum allowed rating expressed as a DanbooruPost::Ratings flag.
 * @return true if the post is unwanted, false otherwise.
 * @author Luca Beltrame (lbeltrame@kde.org)
 */
bool isPostBlacklisted(DanbooruPost *post, QSet<QString> blacklist, Danbooru::Ratings maxRating);

void fixPostUrl(DanbooruPost *post, QUrl boardUrl);

QList<QVariantMap> parseResult(const QByteArray &data, Danbooru::ApiType type, Danbooru::Request request, bool *result);
QList<QVariantMap> parseResult(const QXmlStreamAttributes &data, Danbooru::ApiType type, bool *result);

QVariantMap extractPostData(const QVariant &data, ApiType type);
QVariantMap extractPoolData(const QVariant &data, ApiType type);
QVariantMap extractTagData(const QVariant &data, ApiType type);

const QMap< QString, Danbooru::Rating > initRatingMap();

}

#endif // UTILS_H
